
import datetime

__version__ = '1.0.5'
__author__ = 'Nicola Tudino'
__author_email__ = 'tudo75@gmail.com'
__website_url__ = 'https://github.com/tudo75/nvidia-fan-control'
__license__ = 'GNU General Public License v3.0'
__copyright__ = 'Copyright © 2021-%d %s' % (datetime.date.today().year, __author__)